class Tree extends DNA {

  // Properties
  public PVector pos;
  public int age;
  public boolean alive = true;
  public int timeUntilDeath = -1;
  public int timeUntilDeathCached = 1;
  private int seedCount = 10;
  private int spawnRadiusBase = 30;
  private int spawnAge = 180; // for the moment this is fixed
  public ArrayList<Float> fertGenes;
  
  public boolean fertilized = false;


  // Constructors
  Tree () {
    pos = new PVector(random(width), random(height));
    createGenes();
  }
  Tree (ArrayList<Float> oldGenes) {
    pos = new PVector(random(width), random(height));
    createGenes(oldGenes);
  }
  Tree (ArrayList<Float> oldGenes, boolean direct) {
    pos = new PVector(random(width), random(height));
    createGenes(oldGenes, direct);
  }
  Tree (float x, float y) {
    pos = new PVector(x, y);
    createGenes();
  }
  Tree (float x, float y, ArrayList<Float> oldGenes) {
    pos = new PVector(x, y);
    createGenes(oldGenes);
  }
  Tree (float x, float y, ArrayList<Float> oldGenes, boolean direct) {
    pos = new PVector(x, y);
    createGenes(oldGenes, direct);
  }
  
  
  private color treeColor () {
    if (timeUntilDeath > 0) {
      return color(genes.get(0), 50 + 100 * timeUntilDeath/timeUntilDeathCached, 0);

    }
    return color(genes.get(0), 150, 0);
  }
  private float treeSize () {
    if (timeUntilDeath > 0) {
      return constrain(10 + age*0.015, 0, 40) * timeUntilDeath/timeUntilDeathCached;
    }
    return constrain(10 + age*0.015, 0, 40);
  }
  private float spawnRadius () {
    return spawnRadiusBase + constrain(age/100,0,20);
  }
  
  
  public void display () {
    fill(treeColor());
    ellipse(pos.x, pos.y, treeSize(), treeSize());
  }
  
  public void step() {
    age += 1;
    if (timeUntilDeath < 0) {
      if ( age / spawnAge * increasingChance(age/spawnAge, 200) > random(0, 200)) {
        die();
      }
    } else if (timeUntilDeath > 0) {
      timeUntilDeath -= 1;
      //println("dying");
    } else if (timeUntilDeath == 0) {
      die(true);
    }
    
    if (age % spawnAge == 0) {
      spawn();
    }
    
  }
  
  public void fertilize (ArrayList<Float> newGenes) {
    fertilized = true;
    fertGenes = newGenes;
  }
  
  private void spawn() {
    if (fertilized) {
      for (int i = 0; i < seedCount; i++) {
        if (fertGenes != null) {
          seeds.add(new Seed(pos.x+random(-spawnRadius(),spawnRadius()), pos.y+random(-spawnRadius(), spawnRadius()), fertGenes, genes));
        } else {
          seeds.add(new Seed(pos.x+random(-spawnRadius(),spawnRadius()), pos.y+random(-spawnRadius(), spawnRadius()), genes));
        }
      }
      fertilized = false;
    } else {
      for (int i = 0; i < seedCount; i++) {
        PVector newPos = new PVector(random(-spawnRadius(),spawnRadius()), random(-spawnRadius(),spawnRadius()));
        if (pow(pos.x - newPos.x,2) + pow(pos.y - newPos.y,2) < 36) {
          newPos.x += 6.1; 
          newPos.y += 6.1;
        }
        pollen.add(new Pollen(pos.x + newPos.x, pos.y + newPos.y, genes));
      }
    }
  }

  public void die () {
    timeUntilDeath = (int)random(4,8)*365;
    timeUntilDeathCached = timeUntilDeath;
  }
  public void die (int days) {
    timeUntilDeath = days;
    timeUntilDeathCached = timeUntilDeath;
  }
  public void die (boolean instantDeath)
  {
    if (instantDeath) {
      alive = false;
    } else {
      die();
    }
  }
  
}

