import controlP5.*;
import java.util.Iterator;

ArrayList<Tree> trees;
ArrayList<Seed> seeds;
ArrayList<Pollen> pollen;

int time = 0;

ControlP5 cp5;
Slider sliderTime;
Range sliderResKiller;

PFont font;

void setup () {
  size(1280,720);
  frameRate(24);
  
  randomSeed(millis());
  trees = new ArrayList<Tree>();
  //trees.add(new Tree(width/2, height/2));
  seeds = new ArrayList<Seed>();
  pollen = new ArrayList<Pollen>();
  
  cp5 = new ControlP5(this);
  sliderTime = cp5.addSlider("TimeX")
                         .setPosition(30,height-60)
                         .setValue(30)
                         .setSize(200,30)
                         .setRange(1,60);
    sliderResKiller = cp5.addRange("rangeController")
                         // disable broadcasting since setRange and setRangeValues will trigger an event
                         .setBroadcast(false) 
                         .setPosition(370,height-60)
                         .setValue(0)
                         .setSize(200,30)
                         .setRange(0,255)
                         .setHandleSize(20)
                         .setRangeValues(0,255)
                         // after the initialization we turn broadcast back on again
                         .setBroadcast(true);
                         
  font = createFont("Helvetica",48);                   
}

float envResReqMin = 0;  // temp test
float envResReqMax = 0;  // temp test

void draw() {

  envResReqMin = sliderResKiller.getArrayValue(0);
  envResReqMax = sliderResKiller.getArrayValue(1);
  for (int i = 0; i < (int)sliderTime.getValue(); i++) {
    timeStep();
  }
  background(200);
  noStroke();
  /*for (Seed s : seeds) {
     s.display();
  }*/
  for (Tree t : trees) {
     t.display();
  }
  
  fill(255);
  text((int)(time/365-0.5)+" Years",width-100,height-35);
  
}

void timeStep() {
  for (Iterator<Pollen> it = pollen.iterator(); it.hasNext(); ) {
    Pollen p = it.next();
      for (Tree t: trees) {
        if(t.age > 365 && !t.fertilized && pow(t.pos.x - p.pos.x, 2) + pow(t.pos.y - p.pos.y, 2) < pow(p.fertilisationRadius, 2)) {
          t.fertilize(p.genes);
        }
      }
      it.remove();
  }
  for (Iterator<Tree> it = trees.iterator(); it.hasNext(); ) {
    Tree t = it.next();
    if (t.alive && (t.timeUntilDeath > 0 || (envResReqMin <= t.genes.get(0) && t.genes.get(0) <= envResReqMax))) {
      t.step();
    } else if (t.alive && t.timeUntilDeath < 0 && (t.genes.get(0) < envResReqMin || envResReqMax < t.genes.get(0))) { 
      t.die(365);
    } else {
      it.remove();
    }
  }
  for (Iterator<Seed> it = seeds.iterator(); it.hasNext(); ) {
    Seed s = it.next();
    
    if (s.alive && isInScreen(s.pos)) {
      s.step();
    } else {
      it.remove();
    }
  }
  time += 1;
  if(time % 365 == 0) { 
    println("Year: "+time / 365);
  }
}

void keyPressed () {
  if ( key == 's') {
    ArrayList<Float> genes = new ArrayList<Float>();
      genes.add(0,random(envResReqMin+1,envResReqMax-1));
    for (int i = 0; i < 25; i++) {
      seeds.add(new Seed(mouseX+random(-50,50), mouseY+random(-50,50), genes));
    }
  } else if ( key == 't') {
    trees.add(new Tree(mouseX+random(-50,50), mouseY+random(-50,50)));
  }
}


private boolean isInScreen (PVector p) {
  return 0 <= p.x && p.x <= width && 0 <= p.y && p.y <= height;
}


public float increasingChance (float current, float max) {
  float returnVal = 1;
  for (int i = 0; i < constrain(max-current/10, 0, 14); i++) {
    if (returnVal == 0) return 0.0;
    returnVal *= random(0.1,1);
  }
  return returnVal;
}
